@@ -1,32 +1,20 @@
-/* This file is part of the KDE libraries
-    Copyright (c) 2008 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2008 David Faure <faure@kde.org>
 
-    This library is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2 of the License or ( at
-    your option ) version 3 or, at the discretion of KDE e.V. ( which shall
-    act as a proxy as in section 14 of the GPLv3 ), any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
 #include <QDebug>
 #include <QDir>
-#include <kconfiggroup.h>
-#include <kdesktopfile.h>
+#include <KConfigGroup>
+#include <KDesktopFile>
 #include <kmimetypetrader.h>
 #include <kservicefactory_p.h>
-#include <qtemporarydir.h>
-#include <qtemporaryfile.h>
-#include <qtest.h>
+#include <QTemporaryDir>
+#include <QTemporaryFile>
+#include <QTest>
+#include "setupxdgdirs.h"
 #include "kmimeassociations_p.h"
 #include <kbuildsycoca_p.h>
 #include <ksycoca.h>
@@ -43,7 +31,7 @@ public:
     FakeServiceFactory(KSycoca *db) : KServiceFactory(db) {}
     ~FakeServiceFactory();
 
-    KService::Ptr findServiceByMenuId(const QString &name) Q_DECL_OVERRIDE
+    KService::Ptr findServiceByMenuId(const QString &name) override
     {
         //qDebug() << name;
         KService::Ptr result = m_cache.value(name);
@@ -54,7 +42,7 @@ public:
         //qDebug() << name << result.data();
         return result;
     }
-    KService::Ptr findServiceByDesktopPath(const QString &name) Q_DECL_OVERRIDE
+    KService::Ptr findServiceByDesktopPath(const QString &name) override
     {
         KService::Ptr result = m_cache.value(name); // yeah, same cache, I don't care :)
         if (!result) {
@@ -72,19 +60,23 @@ static bool offerListHasService(const KService::List &offers,
                                 const QString &entryPath,
                                 bool expected /* if set, show error if not found */)
 {
+    // ksycoca resolves to canonical paths, so do it here as well
+    const QString realPath = QFileInfo(entryPath).canonicalFilePath();
+    Q_ASSERT(!realPath.isEmpty());
+
     bool found = false;
-    Q_FOREACH (const KService::Ptr &serv, offers) {
-        if (serv->entryPath() == entryPath) {
+    for (const KService::Ptr &serv : offers) {
+        if (serv->entryPath() == realPath) {
             if (found) {  // should be there only once
-                qWarning("ERROR: %s was found twice in the list", qPrintable(entryPath));
+                qWarning("ERROR: %s was found twice in the list", qPrintable(realPath));
                 return false; // make test fail
             }
             found = true;
         }
     }
     if (!found && expected) {
-        qWarning() << "ERROR:" << entryPath << "not found in offer list. Here's the full list:";
-        Q_FOREACH (const KService::Ptr &serv, offers) {
+        qWarning() << "ERROR:" << realPath << "not found in offer list. Here's the full list:";
+        for (const KService::Ptr &serv : offers) {
             qDebug() << serv->entryPath();
         }
     }
@@ -114,9 +106,10 @@ class KMimeAssociationsTest : public QObject
 private Q_SLOTS:
     void initTestCase()
     {
-        QStandardPaths::enableTestMode(true);
-        qputenv("XDG_CURRENT_DESKTOP", "KDE");
-
+        setupXdgDirs();
+        QStandardPaths::setTestModeEnabled(true);
+        // The Plasma bit makes no sense, but this is just to test that this is treated as a colon-separated list
+        qputenv("XDG_CURRENT_DESKTOP", "KDE:Plasma");
 
         m_localConfig = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/');
         QDir(m_localConfig).removeRecursively();
@@ -160,6 +153,9 @@ private Q_SLOTS:
         fakeHtmlApplicationPrefixed = m_localApps + "fakepfx/fakehtmlapplicationpfx.desktop";
         writeAppDesktopFile(fakeHtmlApplicationPrefixed, QStringList() << QStringLiteral("text/html"));
 
+        fakeOktetaApplication = m_localApps + "fakeoktetaapplication.desktop";
+        writeAppDesktopFile(fakeOktetaApplication, QStringList() << QStringLiteral("application/octet-stream"));
+
         // Update ksycoca in ~/.qttest after creating the above
         runKBuildSycoca();
 
@@ -174,7 +170,7 @@ private Q_SLOTS:
 #if 0
         const KService::List lst = KService::allServices();
         QVERIFY(!lst.isEmpty());
-        Q_FOREACH (const KService::Ptr &serv, lst) {
+        for (const KService::Ptr &serv : lst) {
             qDebug() << serv->entryPath() << serv->storageId() /*<< serv->desktopEntryName()*/;
         }
 #endif
@@ -185,20 +181,25 @@ private Q_SLOTS:
         m_mimeAppsFileContents = "[Added Associations]\n"
                                  "image/jpeg=fakejpegapplication.desktop;\n"
                                  "text/html=fakehtmlapplication.desktop;fakehtmlapplicationpfx.desktop;\n"
-                                 "text/plain=faketextapplication.desktop;fakepfx-faketextapplicationpfx.desktop;gvim.desktop;wine.desktop;idontexist.desktop;\n"
+                                 "text/plain=fakepfx-faketextapplicationpfx.desktop;gvim.desktop;wine.desktop;idontexist.desktop;\n"
                                  // test alias resolution
                                  "application/x-pdf=fakejpegapplication.desktop;\n"
                                  // test x-scheme-handler (#358159) (missing trailing ';' as per xdg-mime bug...)
                                  "x-scheme-handler/mailto=faketextapplication.desktop\n"
+                                 // test association with octet-stream (#425154)
+                                 "application/octet-stream=fakeoktetaapplication.desktop\n"
                                  "[Added KParts/ReadOnlyPart Associations]\n"
                                  "text/plain=katepart.desktop;\n"
                                  "[Removed Associations]\n"
                                  "image/jpeg=firefox.desktop;\n"
-                                 "text/html=gvim.desktop;abiword.desktop;\n";
+                                 "text/html=gvim.desktop;abiword.desktop;\n"
+                                 "[Default Applications]\n"
+                                 "text/plain=faketextapplication.desktop;second-faketextapplicationpfx.desktop\n";
         // Expected results
         preferredApps[QStringLiteral("image/jpeg")] << QStringLiteral("fakejpegapplication.desktop");
         preferredApps[QStringLiteral("application/pdf")] << QStringLiteral("fakejpegapplication.desktop");
         preferredApps[QStringLiteral("text/plain")] << QStringLiteral("faketextapplication.desktop")
+                                    << QStringLiteral("second-faketextapplicationpfx.desktop")
                                     << QStringLiteral("fakepfx-faketextapplicationpfx.desktop")
                                     << QStringLiteral("gvim.desktop");
         preferredApps[QStringLiteral("text/x-csrc")] << QStringLiteral("faketextapplication.desktop")
@@ -208,6 +209,7 @@ private Q_SLOTS:
                                    << QStringLiteral("fakepfx-fakehtmlapplicationpfx.desktop");
         preferredApps[QStringLiteral("application/msword")] << QStringLiteral("fakecsrcmswordapplication.desktop");
         preferredApps[QStringLiteral("x-scheme-handler/mailto")] << QStringLiteral("faketextapplication.desktop");
+        preferredApps[QStringLiteral("text/x-python")] << QStringLiteral("faketextapplication.desktop");
         removedApps[QStringLiteral("image/jpeg")] << QStringLiteral("firefox.desktop");
         removedApps[QStringLiteral("text/html")] << QStringLiteral("gvim.desktop") << QStringLiteral("abiword.desktop");
 
@@ -243,15 +245,17 @@ private Q_SLOTS:
                 end = preferredApps.constEnd(); it != end; ++it) {
             const QString mime = it.key();
             // The data for derived types and aliases isn't for this test (which only looks at mimeapps.list)
-            if (mime == QLatin1String("text/x-csrc") || mime == QLatin1String("application/msword")) {
+            if (mime == QLatin1String("text/x-csrc")
+                    || mime == QLatin1String("text/x-python")
+                    || mime == QLatin1String("application/msword")) {
                 continue;
             }
             const QList<KServiceOffer> offers = offerHash.offersFor(mime);
-            Q_FOREACH (const QString &service, it.value()) {
+            for (const QString &service : it.value()) {
                 KService::Ptr serv = KService::serviceByStorageId(service);
                 if (serv && !offersContains(offers, serv)) {
                     qDebug() << "expected offer" << serv->entryPath() << "not in offers for" << mime << ":";
-                    Q_FOREACH (const KServiceOffer &offer, offers) {
+                    for (const KServiceOffer &offer : offers) {
                         qDebug() << offer.service()->storageId();
                     }
                     QFAIL("offer does not have servicetype");
@@ -263,7 +267,7 @@ private Q_SLOTS:
                 end = removedApps.constEnd(); it != end; ++it) {
             const QString mime = it.key();
             const QList<KServiceOffer> offers = offerHash.offersFor(mime);
-            Q_FOREACH (const QString &service, it.value()) {
+            for (const QString &service : it.value()) {
                 KService::Ptr serv = KService::serviceByStorageId(service);
                 if (serv && offersContains(offers, serv)) {
                     //qDebug() << serv.data() << serv->entryPath() << "does not have" << mime;
@@ -306,12 +310,12 @@ private Q_SLOTS:
         parser.parseMimeAppsList(fileName, 1050); // += 50 is correct.
 
         QList<KServiceOffer> offers = offerHash.offersFor(QStringLiteral("image/jpeg"));
-        qStableSort(offers); // like kbuildservicefactory.cpp does
+        std::stable_sort(offers.begin(), offers.end()); // like kbuildservicefactory.cpp does
         const QStringList expectedJpegApps = preferredApps[QStringLiteral("image/jpeg")];
         QCOMPARE(assembleOffers(offers), expectedJpegApps);
 
         offers = offerHash.offersFor(QStringLiteral("text/html"));
-        qStableSort(offers);
+        std::stable_sort(offers.begin(), offers.end());
         QStringList textHtmlApps = preferredApps[QStringLiteral("text/html")];
         if (KService::serviceByStorageId(QStringLiteral("firefox.desktop"))) {
             textHtmlApps.append(QStringLiteral("firefox.desktop"));
@@ -320,7 +324,7 @@ private Q_SLOTS:
         QCOMPARE(assembleOffers(offers), textHtmlApps);
 
         offers = offerHash.offersFor(QStringLiteral("image/png"));
-        qStableSort(offers);
+        std::stable_sort(offers.begin(), offers.end());
         QCOMPARE(assembleOffers(offers), QStringList() << QStringLiteral("fakejpegapplication.desktop"));
     }
 
@@ -330,6 +334,7 @@ private Q_SLOTS:
 
         // Test a trader query
         KService::List offers = KMimeTypeTrader::self()->query(QStringLiteral("image/jpeg"));
+        QVERIFY(!offers.isEmpty());
         //qDebug() << m_mimeAppsFileContents;
         //qDebug() << "preferred apps for jpeg: " << preferredApps.value("image/jpeg");
         //for (int i = 0; i < offers.count(); ++i) {
@@ -415,6 +420,16 @@ private Q_SLOTS:
         // #178560: Removing ark from interface/x-winamp-skin didn't work
         // Using application/x-kns (another zip-derived mimetype) nowadays.
         const QString mime = QStringLiteral("application/x-kns");
+
+        // That mimetype comes from kcoreaddons, let's make sure it's properly installed
+        {
+            QMimeDatabase db;
+            QMimeType mime = db.mimeTypeForName(QStringLiteral("application/x-kns"));
+            QVERIFY(mime.isValid());
+            QCOMPARE(mime.name(), QStringLiteral("application/x-kns"));
+            QVERIFY(mime.inherits(QStringLiteral("application/zip")));
+        }
+
         KService::List offers = KMimeTypeTrader::self()->query(mime);
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
 
@@ -477,7 +492,7 @@ private:
 
     static bool offersContains(const QList<KServiceOffer> &offers, KService::Ptr serv)
     {
-        Q_FOREACH (const KServiceOffer &offer, offers) {
+        for (const KServiceOffer &offer : offers) {
             if (offer.service()->storageId() == serv->storageId()) {
                 return true;
             }
@@ -487,7 +502,7 @@ private:
     static QStringList assembleOffers(const QList<KServiceOffer> &offers)
     {
         QStringList lst;
-        Q_FOREACH (const KServiceOffer &offer, offers) {
+        for (const KServiceOffer &offer : offers) {
             lst.append(offer.service()->storageId());
         }
         return lst;
@@ -495,7 +510,7 @@ private:
     static QStringList assembleServices(const QList<KService::Ptr> &services, int maxCount = -1)
     {
         QStringList lst;
-        Q_FOREACH (const KService::Ptr &service, services) {
+        for (const KService::Ptr &service : services) {
             lst.append(service->storageId());
             if (maxCount > -1 && lst.count() == maxCount) {
                 break;
@@ -527,6 +542,7 @@ private:
     QString fakeHtmlApplication;
     QString fakeHtmlApplicationPrefixed;
     QString fakeArkApplication;
+    QString fakeOktetaApplication;
 
     ExpectedResultsMap preferredApps;
     ExpectedResultsMap removedApps;
