@@ -1,25 +1,13 @@
-/* This file is part of the KDE libraries
-    Copyright (c) 1997,1998 Matthias Kalle Dalheimer <kalle@kde.org>
-    Copyright (c) 1999 Espen Sand <espen@kde.org>
-    Copyright (c) 2000-2004 Frerich Raabe <raabe@kde.org>
-    Copyright (c) 2003,2004 Oswald Buddenhagen <ossi@kde.org>
-    Copyright (c) 2006 Thiago Macieira <thiago@kde.org>
-    Copyright (C) 2008 Aaron Seigo <aseigo@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1997, 1998 Matthias Kalle Dalheimer <kalle@kde.org>
+    SPDX-FileCopyrightText: 1999 Espen Sand <espen@kde.org>
+    SPDX-FileCopyrightText: 2000-2004 Frerich Raabe <raabe@kde.org>
+    SPDX-FileCopyrightText: 2003, 2004 Oswald Buddenhagen <ossi@kde.org>
+    SPDX-FileCopyrightText: 2006 Thiago Macieira <thiago@kde.org>
+    SPDX-FileCopyrightText: 2008 Aaron Seigo <aseigo@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "ktoolinvocation.h"
@@ -253,11 +241,11 @@ void KToolInvocation::invokeBrowser(const QString &url, const QByteArray &startu
     args << url;
     QString error;
 
-    // This method should launch a webbrowser, preferably without doing a mimetype
+    // This method should launch a webbrowser, preferably without doing a MIME type
     // check first, like KRun (i.e. kde-open) would do.
 
     // In a KDE session, honour BrowserApplication if set, otherwise use preferred app for text/html if any,
-    // otherwise xdg-open, otherwise kde-open (which does a mimetype check first though).
+    // otherwise xdg-open, otherwise kde-open (which does a MIME type check first though).
 
     // Outside KDE, call xdg-open if present, otherwise fallback to the above logic.
 
@@ -372,7 +360,7 @@ void KToolInvocation::invokeTerminal(const QString &command,
 
     QString error;
     if (self()->startServiceInternal("kdeinit_exec_with_workdir",
-                                     cmd, cmdTokens, envs, &error, nullptr, nullptr, startup_id, false, workdir)) {
+                                     cmd, cmdTokens, &error, nullptr, nullptr, startup_id, false, workdir, envs)) {
         KMessage::message(KMessage::Error,
                           i18n("Could not launch the terminal client:\n\n%1", error),
                           i18n("Could not launch Terminal Client"));
